@@ -138,11 +138,11 @@
 #' The function offers either the non-linear least squares estimation via the
 #' \code{\link[stats]{nls}} function (Drabinova & Martinkova, 2017; Hladka &
 #' Martinkova, 2020), the maximum likelihood method with the \code{"L-BFGS-B"}
-#' algorithm with constraints via the \code{\link[stats]{optim}} function (Hladka &
-#' Martinkova, 2020), the maximum likelihood method with the EM algorithm (Hladka,
-#' Martinkova, & Brabec, 2024), the maximum likelihood method with the algorithm
-#' based on parametric link function (PLF, the default option; Hladka, Martinkova,
-#' & Brabec, 2024), or the maximum likelihood method with the iteratively
+#' algorithm with constraints via the \code{\link[stats]{optim}} function
+#' (Hladka & Martinkova, 2020), the maximum likelihood method with the EM
+#' algorithm (Hladka, Martinkova, & Brabec, 2025), the maximum likelihood method
+#' with the algorithm based on parametric link function (Hladka, Martinkova, &
+#' Brabec, 2025), or the maximum likelihood method with the iteratively
 #' reweighted least squares algorithm via the \code{\link[stats]{glm}} function.
 #'
 #' @return A list with the following arguments:
@@ -196,7 +196,7 @@
 #' models for DIF and DDF detection. The R Journal, 12(1), 300--323,
 #' \doi{10.32614/RJ-2020-014}.
 #'
-#' Hladka, A., Martinkova, P., & Brabec, M. (2024). New iterative algorithms
+#' Hladka, A., Martinkova, P., & Brabec, M. (2025). New iterative algorithms
 #' for estimation of item functioning. Journal of Educational and Behavioral
 #' Statistics. Online first, \doi{10.3102/10769986241312354}.
 #'
@@ -306,6 +306,9 @@ NLR <- function(Data, group, model, constraints = NULL, type = "all", method = "
   } else if (missing(start) || is.null(start)) {
     start <- startNLR(Data, group, model, match = x, parameterization = parameterization)
   } else {
+    if (all(sapply(1:m, function(i) all(M[[i]]$M1$parameters %in% names(start[[i]]))))) {
+      start <- lapply(1:m, function(i) start[[i]][M[[i]]$M1$parameters])
+    }
     if (!all(sapply(1:m, function(i) length(start[[i]]) == length(M[[i]]$M1$parameters))) ||
       !all(sapply(1:m, function(i) all(sort(names(start[[i]])) == sort(M[[i]]$M1$parameters))))) {
       warning("Invalid names of item parameters in 'start'. Initial values are calculated with the 'startNLR' function.", call. = FALSE)
@@ -599,7 +602,7 @@ You may try increasing the number of recalculations using the `nrBo` argument. "
 
   # parameters
   par.m0 <- lapply(1:m, function(i) {
-    structure(rep(NA, length(M[[i]]$M0$parameters)),
+    pars <- structure(rep(NA, length(M[[i]]$M0$parameters)),
       names = M[[i]]$M0$parameters
     )
   })
@@ -613,6 +616,35 @@ You may try increasing the number of recalculations using the `nrBo` argument. "
 
   names(par.m1) <- names(par.m0) <- names(se.m1) <- names(se.m0) <- names(cov.m0) <- names(cov.m1) <- colnames(Data)
 
+  if (method == "irls") {
+    par.m0 <- lapply(par.m0, function(x) {
+      names(x) <- gsub("g", "b2", gsub("x", "b1", gsub("x:g", "b3", gsub("\\(Intercept\\)", "b0", names(x)))))
+      x
+    })
+    par.m1 <- lapply(par.m1, function(x) {
+      names(x) <- gsub("g", "b2", gsub("x", "b1", gsub("x:g", "b3", gsub("\\(Intercept\\)", "b0", names(x)))))
+      x
+    })
+    se.m0 <- lapply(se.m0, function(x) {
+      names(x) <- gsub("g", "b2", gsub("x", "b1", gsub("x:g", "b3", gsub("\\(Intercept\\)", "b0", names(x)))))
+      x
+    })
+    se.m1 <- lapply(se.m1, function(x) {
+      names(x) <- gsub("g", "b2", gsub("x", "b1", gsub("x:g", "b3", gsub("\\(Intercept\\)", "b0", names(x)))))
+      x
+    })
+    cov.m0 <- lapply(cov.m0, function(x) {
+      tmp <- gsub("g", "b2", gsub("x", "b1", gsub("x:g", "b3", gsub("\\(Intercept\\)", "b0", rownames(x)))))
+      rownames(x) <- colnames(x) <- tmp
+      x
+    })
+    cov.m1 <- lapply(cov.m1, function(x) {
+      tmp <- gsub("g", "b2", gsub("x", "b1", gsub("x:g", "b3", gsub("\\(Intercept\\)", "b0", rownames(x)))))
+      rownames(x) <- colnames(x) <- tmp
+      x
+    })
+  }
+
   # test
   n0 <- sapply(1:m, function(i) length(M[[i]]$M0$parameters))
   n1 <- sapply(1:m, function(i) length(M[[i]]$M1$parameters))
@@ -635,6 +667,7 @@ You may try increasing the number of recalculations using the `nrBo` argument. "
       function(l) (1 - pf(Fval[l], df[l, 1], df[l, 2]))
     )
   } else if (test == "LR") {
+    df <- df[, 1]
     pval <- LRval <- rep(NA, m)
 
     LRval[conv] <- sapply(
@@ -646,6 +679,7 @@ You may try increasing the number of recalculations using the `nrBo` argument. "
       function(l) (1 - pchisq(LRval[l], df[l]))
     )
   } else if (test == "W") {
+    df <- df[, 1]
     pval <- Wval <- rep(NA, m)
 
     Wval[conv] <- sapply(
@@ -665,6 +699,31 @@ You may try increasing the number of recalculations using the `nrBo` argument. "
   # adjusted p-values
   adjusted.pval <- p.adjust(pval, method = p.adjust.method)
 
+  # adjusting asymptote parameters
+  for (i in 1:m) {
+    if ("cF" %in% names(par.m0[[i]]) | "dF" %in% names(par.m0[[i]])) {
+      adj.par.m0 <- .deltamethod.NLR.asymptotes(
+        par = par.m0[[i]], cov = cov.m0[[i]],
+        conv = (i %in% conv0),
+        cov_fail = (i %in% items.cov.fail0)
+      )
+      par.m0[[i]] <- adj.par.m0[["par"]]
+      cov.m0[[i]] <- adj.par.m0[["cov"]]
+      se.m0[[i]] <- adj.par.m0[["se"]]
+    }
+
+    if ("cF" %in% names(par.m1[[i]]) | "dF" %in% names(par.m1[[i]])) {
+      adj.par.m1 <- .deltamethod.NLR.asymptotes(
+        par = par.m1[[i]], cov = cov.m1[[i]],
+        conv = (i %in% conv1),
+        cov_fail = (i %in% items.cov.fail1)
+      )
+      par.m1[[i]] <- adj.par.m1[["par"]]
+      cov.m1[[i]] <- adj.par.m1[["cov"]]
+      se.m1[[i]] <- adj.par.m1[["se"]]
+    }
+  }
+
   results <- list(
     Sval = switch(test,
       "F" = Fval,
@@ -684,69 +743,150 @@ You may try increasing the number of recalculations using the `nrBo` argument. "
 
 #' @noRd
 .deltamethod.NLR.is2irt <- function(par, cov, conv, cov_fail) {
+  # for Rasch model, IS and IRT is the same
+  if (length(par) == 1L && names(par) == "b0") {
+    names(par) <- "b"
+    cov <- as.matrix(cov) # to be able to set dimnames
+    dimnames(cov) <- list("b", "b")
+    return(list(par = par, cov = cov, se = sqrt(cov)))
+  }
 
-    par_names <- which(c("b0", "b1", "b2", "b3", "c", "cR", "cF", "d", "dR", "dF") %in% names(par))
+  par_names <- which(c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif") %in% names(par))
 
-    new_order <- sapply(c("a", "b", "aDif", "bDif", "c", "cR", "cF", "d", "dR", "dF"), function(x) {
-      which(x == c("b", "a", "bDif", "aDif", "c", "cR", "cF", "d", "dR", "dF")[par_names])
-    })
-    new_order <- unlist(new_order[sapply(new_order, function(x) length(x) > 0)])
+  new_order <- sapply(c("a", "b", "aDif", "bDif", "c", "cDif", "d", "dDif"), function(x) {
+    which(x == c("b", "a", "bDif", "aDif", "c", "cDif", "d", "dDif")[par_names])
+  })
+  new_order <- unlist(new_order[sapply(new_order, function(x) length(x) > 0)])
+  # TODO: zkontrolovat jestli funguje pro vsechny modely b1 = 1
+  par_tmp <- setNames(
+    c(c(0, 1), rep(0, 4), rep(1, 2)),
+    c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif")
+  )
+  par_tmp[par_names] <- par
+  par_new <- setNames(
+    c(
+      -par_tmp[1] / par_tmp[2],
+      par_tmp[2],
+      (par_tmp[1] * par_tmp[4] - par_tmp[2] * par_tmp[3]) / (par_tmp[2] * (par_tmp[2] + par_tmp[4])),
+      par_tmp[4],
+      par_tmp[5],
+      par_tmp[6],
+      par_tmp[7],
+      par_tmp[8]
+    ),
+    c("b", "a", "bDif", "aDif", "c", "cDif", "d", "dDif")
+  )[par_names]
 
-    par_tmp <- setNames(
-      c(rep(0, 7), rep(1, 3)),
-      c("b0", "b1", "b2", "b3", "c", "cR", "cF", "d", "dR", "dF")
+  if (cov_fail | !conv) {
+    npar <- length(par_new)
+    se_new <- setNames(rep(NA, npar), names(par_new)[new_order])
+    cov_new <- matrix(
+      NA,
+      ncol = npar, nrow = npar,
+      dimnames = list(
+        names(par_new)[new_order],
+        names(par_new)[new_order]
+      )
     )
-    par_tmp[par_names] <- par
-    par_new <- setNames(
-      c(
-        -par_tmp[1] / par_tmp[2],
-        par_tmp[2],
-        (par_tmp[1] * par_tmp[4] - par_tmp[2] * par_tmp[3]) / (par_tmp[2] * (par_tmp[2] + par_tmp[4])),
-        par_tmp[4],
-        par_tmp[5],
-        par_tmp[6],
-        par_tmp[7],
-        par_tmp[8],
-        par_tmp[9],
-        par_tmp[10]
+  } else {
+    cov_tmp <- matrix(
+      0,
+      ncol = 8, nrow = 8,
+      dimnames = list(
+        c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif"),
+        c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif")
+      )
+    )
+    cov_tmp[par_names, par_names] <- cov
+    cov_new <- msm::deltamethod(
+      list(
+        ~ -x1 / x2, ~x2, ~ (x1 * x4 - x2 * x3) / (x2 * (x2 + x4)), ~x4,
+        ~x5, ~x6, ~x7, ~x8
       ),
-      c("b", "a", "bDif", "aDif", "c", "cR", "cF", "d", "dR", "dF")
-    )[par_names]
+      par_tmp,
+      cov_tmp,
+      ses = FALSE
+    )[par_names, par_names]
+    cov_new <- matrix(cov_new, nrow = length(par_names), ncol = length(par_names))
+    colnames(cov_new) <- rownames(cov_new) <- c("b", "a", "bDif", "aDif", "c", "cDif", "d", "dDif")[par_names]
+    cov_new <- matrix(cov_new[new_order, new_order],
+      nrow = length(par_names), ncol = length(par_names),
+      dimnames = list(rownames(cov_new)[new_order], colnames(cov_new)[new_order])
+    )
+    se_new <- sqrt(diag(cov_new))
+  }
 
-    if (cov_fail | !conv) {
-      npar <- length(par_new)
-      se_new <- setNames(rep(NA, npar), names(par_new)[new_order])
-      cov_new <- matrix(
-        NA,
-        ncol = npar, nrow = npar,
-        dimnames = list(
-          names(par_new)[new_order],
-          names(par_new)[new_order]
-        )
+  return(list(par = par_new[new_order], cov = cov_new, se = se_new))
+}
+
+
+#' @noRd
+.deltamethod.NLR.asymptotes <- function(par, cov, conv, cov_fail) {
+  # this function is executed only when cF or dF is present in parameters
+  if ("c" %in% names(par)) {
+    names(par)[names(par) == "c"] <- "cR"
+    colnames(cov)[colnames(cov) == "c"] <- "cR"
+    rownames(cov)[rownames(cov) == "c"] <- "cR"
+  }
+  if ("d" %in% names(par)) {
+    names(par)[names(par) == "d"] <- "dR"
+    colnames(cov)[colnames(cov) == "d"] <- "dR"
+    rownames(cov)[rownames(cov) == "d"] <- "dR"
+  }
+  par_names <- which(c("b0", "b1", "b2", "b3", "cR", "cF", "dR", "dF") %in% names(par))
+  par_tmp <- setNames(
+    c(c(0, 1), rep(0, 4), rep(1, 2)),
+    c("b0", "b1", "b2", "b3", "cR", "cF", "dR", "dF")
+  )
+  par_tmp[par_names] <- par
+  par_new <- setNames(
+    c(
+      par_tmp[1], # b0
+      par_tmp[2], # b1
+      par_tmp[3], # b2
+      par_tmp[4], # b3
+      par_tmp[5], # c = cR
+      par_tmp[6] - par_tmp[5], # cDif = cF - cR
+      par_tmp[7], # d = dR
+      par_tmp[8] - par_tmp[7] # dDif = dF - dR
+    ),
+    c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif")
+  )[par_names]
+
+  if (cov_fail | !conv) {
+    npar <- length(par_new)
+    se_new <- setNames(rep(NA, npar), names(par_new))
+    cov_new <- matrix(
+      NA,
+      ncol = npar, nrow = npar,
+      dimnames = list(
+        names(par_new),
+        names(par_new)
       )
-    } else {
-      cov_tmp <- matrix(
-        0,
-        ncol = 10, nrow = 10,
-        dimnames = list(
-          c("b0", "b1", "b2", "b3", "c", "cR", "cF", "d", "dR", "dF"),
-          c("b0", "b1", "b2", "b3", "c", "cR", "cF", "d", "dR", "dF")
-        )
+    )
+  } else {
+    cov_tmp <- matrix(
+      0,
+      ncol = 8, nrow = 8,
+      dimnames = list(
+        c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif"),
+        c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif")
       )
-      cov_tmp[par_names, par_names] <- cov
-      cov_new <- msm::deltamethod(
-        list(
-          ~ -x1 / x2, ~x2, ~ (x1 * x4 - x2 * x3) / (x2 * (x2 + x4)), ~x4,
-          ~x5, ~x6, ~x7, ~x8, ~x9, ~x10
-        ),
-        par_tmp,
-        cov_tmp,
-        ses = FALSE
-      )[par_names, par_names]
-      colnames(cov_new) <- rownames(cov_new) <- c("b", "a", "bDif", "aDif", "c", "cR", "cF", "d", "dR", "dF")[par_names]
-      cov_new <- cov_new[new_order, new_order]
-      se_new <- sqrt(diag(cov_new))
-    }
+    )
+    cov_tmp[par_names, par_names] <- cov
+    cov_new <- msm::deltamethod(
+      list(
+        ~x1, ~x2, ~x3, ~x4,
+        ~x5, ~ x6 - x5, ~x7, ~ x8 - x7
+      ),
+      par_tmp,
+      cov_tmp,
+      ses = FALSE
+    )[par_names, par_names]
+    cov_new <- matrix(cov_new, nrow = length(par_names), ncol = length(par_names))
+    colnames(cov_new) <- rownames(cov_new) <- c("b0", "b1", "b2", "b3", "c", "cDif", "d", "dDif")[par_names]
+    se_new <- sqrt(diag(cov_new))
+  }
 
-    return(list(par = par_new[new_order], cov = cov_new, se = se_new))
+  return(list(par = par_new, cov = cov_new, se = se_new))
 }
